
#include <math.h>
#include "nrutil.h"
#include "NR.h"

#define MAXITS 200000
#define EPS      1.0e-7
//#define EPS      1.0e-16
#define EPS_JAC  1.0e-4
//#define EPS_JAC  1.0e-1
#define TOLF 1.0e-12
#define TOLX EPS
#define STPMX 100.0
//#define TOLMIN 1.0e-6
#define TOLMIN 1.0e-13

#define ALF 1.0e-4
//#define TOLX_LNSRCH 1.0e-7
#define TOLX_LNSRCH 1.0e-12

#define FREE_RETURN {      \
  free_dvector(  fvec,1,n); \
  free_dvector(  xold,1,n); \
  free_dvector(     w,1,n); \
  free_dvector(     t,1,n); \
  free_dvector(     s,1,n); \
  free_matrix( r,1,n,1,n); \
  free_matrix(qt,1,n,1,n); \
  free_dvector(     p,1,n); \
  free_dvector(     g,1,n); \
  free_dvector(fvcold,1,n); \
  free_dvector(     d,1,n); \
  free_dvector(     c,1,n); \
  return;  \
}


//===========================================================================
int nn;
double *fvec;
MULTI_FUN nrfuncv;

//void (*nrfuncv)( int n, double v[], double f[] );
//===========================================================================
double fmin( double x[] )
{
  int i;
  double sum;
  ( *nrfuncv )( x, fvec, nn );
  for( sum=0.0, i=1; i<=nn; i++ ){ sum += SQR( fvec[i] ); }
  return( 0.5*sum );
}
//===========================================================================
void fdjac( int n, 
	    double    x[], 
	    double fvec[], 
	    double **df,
	    MULTI_FUN vecfunc
	    )
{

  int i,j;
  double h, temp, *f;
  f=dvector(1,n);
  for( j=1; j<=n; j++ ){

    temp=x[j];
    h=EPS_JAC*fabs(temp);

    if(h==0.0){ h=EPS_JAC; }

    x[j]=temp+h;
    h=x[j]-temp;
    (*vecfunc)(x,f,n);
    x[j] = temp;
    for(i=1;i<=n;i++){ 
      df[i][j]=( f[i]-fvec[i] )/h; 
    }
  }
  free_dvector(f,1,n);
}
//============================================================================
void broydn( double *x,
	     MULTI_FUN vecfunc,
	     const int& n,
	     const double& tolerancef,
	     int* check
	     )
{

  int i, its, j, k, restrt, sing, skip;
  double den, f, fold, stpmax, sum, temp, test, *c, *d, *fvcold;
  double *g, *p, **qt, **r, *s, *t, *w, *xold;

  c      = dvector(1,n);
  d      = dvector(1,n);
  fvcold = dvector(1,n);
  g      = dvector(1,n);
  p      = dvector(1,n);

  qt     = matrix(1,n,1,n);
  r      = matrix(1,n,1,n);
  
  s	 = dvector(1,n);
  t	 = dvector(1,n);
  w	 = dvector(1,n);
  xold	 = dvector(1,n);
  fvec	 = dvector(1,n);

  nn=n;
  nrfuncv=vecfunc;
  f=fmin(x);
  test=0.0;
  
  for( int i=1; i<=n; i++ ){ cout << fvec[i] << endl; }

  for( i=1; i<=n; i++ ){ if( fabs(fvec[i]) > test ){ test = fabs( fvec[i] );}}

  // Initial Guess Already satisfy TOLERANCE //
  if( test < 0.01*TOLF ){ *check=0; FREE_RETURN; }

  // stpmax will be used for the line searches. //
  for( sum=0.0, i=1; i<=n; i++ ){ sum += SQR( x[i] ); }
  stpmax=STPMX*FMAX( sqrt(sum), (double)n );

  restrt=1;
  // Main Part of Iteration //
  for( its=1; its<=MAXITS; its++ ){

    cout << "its=" << its << endl;
    for( i=1; i<=n; i++ ){ 
      cout << "x[" << i << "]=" << x[i] << "\t"
	   << "fvec[" << i << "]=" << fvec[i] << endl; 
    }

    if( restrt ){
      fdjac(n,x,fvec,r,vecfunc); // Initialize or re-initialize Jacobian in r 
      qrdcmp(r,n,c,d,&sing);     // QR decomposition of Jacobian
      if( sing ){ nrerror( "singular Jacobian in broydn" ); }

      // Form Q^T using explicit method //
      for( i=1; i<=n; i++ ){
	for( j=1; j<=n; j++ ){ qt[i][j] = 0.0; }
	qt[i][i]=1.0;
      }
      for( k=1; k<n; k++ ){
	if( c[k] ){
	  for( j=1; j<=n; j++ ){ 
	    sum = 0.0;
	    for( i=k; i<=n; i++ ){ sum += r[i][k]*qt[i][j]; }
	    sum /= c[k];
	    for( i=k; i<=n; i++ ){ qt[i][j] -= sum*r[i][k]; }
	  }
	}
      }

      // Form R explicitly //
      for( i=1; i<=n; i++ ){
	r[i][i]=d[i];
	for( j=1; j<i; j++ ){ r[i][j] = 0.0; }
      }

    }else{ // Carry out Broyden update 

      for( i=1; i<=n; i++ ){ s[i]=x[i]-xold[i]; } // s = delta x
      for( i=1; i<=n; i++ ){                      // t = R * s
	for( sum=0.0, j=i; j<=n; j++ ){ sum += r[i][j]*s[j]; }
	t[i]=sum;
      }
      skip=1;
      for(i=1;i<=n;i++){
	for( sum=0.0, j=1; j<=n; j++ ){ sum += qt[j][i]*t[j]; }
	w[i]=fvec[i]-fvcold[i]-sum;
	if( fabs(w[i]) >= EPS*( fabs(fvec[i])+fabs(fvcold[i]))){ 
	  skip = 0; 
	}else{
	  // Do not update with noisy component of w //
	  w[i]=0.0;
	}
      }
      
      if( !skip ){
	for(i=1; i<=n; i++ ){
	  for( sum=0.0,j=1; j<=n; j++ ){ sum += qt[i][j]*w[j]; }
	  t[i]=sum;
	}

	for( den=0.0, i=1; i<=n; i++ ){ den  += SQR(s[i]); }
	for(          i=1; i<=n; i++ ){ s[i] /= den;       } 
	// Store s/(s*s) in s 

	qrupdt( r, qt, n, t, s ); // Update R and Q^T

	for( i=1; i<=n; i++ ){
	  if( r[i][i]==0.0 ){ nrerror("r singular in broydn" ); }
	  d[i]=r[i][i];
	}
      }// !skip
    }

    // Calculate (grad f) simeq ( Q * R )^T F for the line search
    for( i=1; i<=n; i++ ){
      for( sum=0.0,j=1; j<=n; j++ ){ sum += qt[i][j]*fvec[j]; }
      g[i]=sum;
    }

    for( i=n; i>=1; i-- ){
      for( sum=0.0,j=1; j<=i; j++ ){ sum += r[j][i]*g[j]; }
      g[i]=sum;
    }

    // Sotre x and F
    for( i=1; i<=n; i++ ){
      xold[i]=x[i];
      fvcold[i]=fvec[i];
    }
    fold=f;  // Store f

    // Right Hand side for linear equation is - Q^T F
    for( i=1; i<=n; i++ ){
      for( sum=0.0,j=1; j<=n; j++ ){ sum += qt[i][j]*fvec[j]; }
      p[i] = - sum;
    }

    rsolv(r,n,d,p);
    lnsrch(n,xold,fold,g,p,x,&f, stpmax,check,fmin);
    // lnsrch returns new x and f.
    // It also calculates fvec at the new x when it calls fmin
    test=0.0;
    
    for(i=1;i<=n;i++){ if( fabs( fvec[i] ) > test ){ test = fabs( fvec[i] ); }}

    if( test < TOLF ){
      *check=0;
      FREE_RETURN;
    }

    if( *check ){ 
      if( restrt ){ 
	FREE_RETURN; 
      }else{
	test = 0.0;
	den = FMAX( f, 0.5*n );
	for( i=1; i<=n; i++ ){
	  temp=fabs( g[i] )*FMAX( fabs( x[i] ), 1.0 )/den;
	  if( temp > test ){ test = temp; }
	}
	if( test < TOLMIN ){ 
	  FREE_RETURN; 
	}else{
	  restrt = 1; // Try re-initializing on delta x //
	}
      }

    }else{
      restrt=0;
      test=0.0;
      for( i=1; i<=n; i++ ){
	temp=(fabs( x[i]-xold[i]))/FMAX(fabs(x[i]),1.0 );
	if( temp > test ){ test = temp; }
      }
      if( test < TOLX ){ FREE_RETURN; }
    }
  }
  nrerror( "MAXITS exceeds in broydn" );
  FREE_RETURN;
}
//============================================================================
void lnsrch( int n, double xold[], double fold, double g[], double p[], 
	     double x[], double *f, double stpmax, int *check, 
	     double (*func)( double [] ) 
	     )
{

  int i; 
  double a, alam, alam2, alamin, b, disc, f2;
  double rhs1, rhs2, slope, sum, temp, test, tmplam;

  *check=0;
  for( sum=0.0, i=1; i<=n; i++ ){ sum+=p[i]*p[i]; }
  sum = sqrt(sum);
  if( sum > stpmax ){ for( i=1;i<=n; i++ ){ p[i] *= stpmax/sum; } }
  for( slope=0.0, i=1; i<=n; i++ ){ slope += g[i]*p[i]; }
  if( slope >=0.0 ){ nrerror("Roundoff problem in lnsrch." ); }
  test=0.0;

  for( i=1; i<=n; i++ ){
    temp=fabs( p[i] )/FMAX( fabs( xold[i] ), 1.0 );
    if( temp > test ){ test = temp; }
  }

  alamin=TOLX_LNSRCH/test;
  alam=1.0;

  for(;;){

    for( i=1; i<=n; i++ ){ x[i] = xold[i] + alam * p[i]; }
    *f = (*func)(x);
    if( alam < alamin ){
      for( i=1; i<=n; i++ ){ x[i]=xold[i]; }
      *check=1;
      return;
    }else if( *f <= fold + ALF*alam*slope ){ 
      return; 
    }else{
 
      if( alam == 1.0 ){
	tmplam = - slope/( 2.0 * (*f - fold -slope));
      }else{

	rhs1 = *f -fold -alam*slope;
	rhs2 = f2 -fold -alam2*slope;
	a=( rhs1/(alam*alam)-rhs2/(alam2*alam2) )/(alam-alam2);
	b=( -alam2*rhs1/(alam*alam)+alam*rhs2/(alam2*alam2))/(alam-alam2);

	if( a==0.0 ){ 
	  tmplam = - slope/(2.0*b); 
	}else{
	  disc=b*b-3.0*a*slope;
	  if( disc < 0.0 ){ 
	    tmplam = 0.5*alam;
	  }else if( b<=0.0 ){ 
	    tmplam = ( -b+sqrt(disc) )/(3.0*a);
	  }else{
	    tmplam = - slope/( b+sqrt(disc) );
	  }
	}
	if( tmplam > 0.5*alam ){ tmplam = 0.5*alam; }
      }
    }
    alam2=alam;
    f2 = *f;
    alam = FMAX( tmplam, 0.1*alam);
  }
}
//============================================================================
