#include <math.h>
#include <iostream>
#include "Gauss.h"
#include "QuasiNewton.h"

//=========================================================================
//typedef int (*MULTI_FUN)( double* x, double* f, int N );

int Jacobi_Matrix( double*   &xo, 
		   double**  &J, 
		   MULTI_FUN  fun, 
		   const int &N );

int multi_fun (   double* &x, double* &f, const int &N );
int Write_Result( double* &x, double* &f, const int &N, const double &eps );
//=========================================================================
int main()
{
  int N=2; 
  double eps=1.0e-10;
  double *x, *f, **H;
  x =new double[N];
  f =new double[N];
  H = new double* [N];
  H[0] = new double[N];
  H[1] = new double[N];
  
  x[0]=3.0, x[1]=3.0;
  //=======================================
  Jacobi_Matrix( x, H, multi_fun, N );

  GJE0( H, N );

  multi_fun( x, f, N );
  quasi_Newton_method( x, f, H, multi_fun, N, eps );
  //=======================================

  Write_Result( x, f, N, eps );

  return(0);
}
//=========================================================================
int multi_fun (   double* &x, double* &f, const int &N )
{
  f[0] = x[0]*x[0] + x[1]*x[1] -5.0;
  f[1] = x[0]+2.0*x[1];
  return( 0 );
}
//=========================================================================
int Jacobi_Matrix( double* &xo, double** &J, MULTI_FUN fun, const int &N )
{
  int i,j;
  double dx = 1.0e-2;
  // J[i][j] // i: runs for functions, j: runs for variables

  double *x1 = new double[N];
  double *fo = new double[N];
  double *fd = new double[N];

  double inv_dx  = 1.0/dx;

  fun( xo, fo, N );

  for( j=0; j<N; j++ ){

    for( int jj=0; jj<N; jj++ ){
      x1[jj] = xo[jj];
    }
    x1[j] = xo[j] + dx; 

    fun( x1, fd, N );

    for( i=0; i<N; i++ ){
      J[i][j] = ( fd[i] - fo[i] )*inv_dx;
    }
  }

  //delete [] x1;
  //delete [] fo;
  //delete [] fd;
  return(0);
}
//=========================================================================
int Write_Result( double* &x, double* &f, const int &N, const double &eps )
{
  int i,j;
  printf("\n");
  printf("***********************\n");
  for( j=0; j<N; j++ ){ 
    printf("x[%d]=%17.17e ", j, x[j] );
  }
  printf("\n");

  for( j=0; j<N; j++ ){ 
    printf("F[%d]=%17.17e ", j, f[j] );
  }
  printf("\n");
  printf("***********************\n");
  printf("This is solution ! for eps = %17.17e\n",eps);
  printf("\n");
  return(0);
}
//=========================================================================
