#include <math.h>
#include "Gauss.h"
#include "QuasiNewton.h"
#define CHI 4
//=========================================================================
//typedef int (*MULTI_FUN  )( double* &x, double* &f, const int &N );

int Jacobi_Matrix( double* xo, 
		   double ** J, 
		   MULTI_FUN fun, 
		   int N );

int multi_fun (   double* &x, double* &f, const int &N );
int Write_Result( double* &x, double* &f, const int &N, const double &eps );
//=========================================================================
int main()
{
  int N=2; double eps=1.0e-10;
  double *x, *f, **H;
  x = new double    [N];
  f = new double    [N];
  H = new double*   [N];
  H[0] = new double [N];
  H[1] = new double [N];
  
  x[0]=0.48, x[1] = 0.52;
  //=======================================
  Jacobi_Matrix( x, H, multi_fun, N );
  GJE0( H, N );
  multi_fun( x, f, N );
  quasi_Newton_method( x, f, H, multi_fun, N, eps );
  //=======================================
  Write_Result( x, f, N, eps );

  return(0);
}
//=========================================================================
double mu( const double& x )
{
  double chi = CHI;
  // f =  x log x + (1-x)log(1-x) - x*x*chi
  double mu  = log(x) - log(1-x) - 2*x*chi;
  return(mu);
}
//=========================================================================
double pi( const double& x )
{
  double chi = CHI;
  //pi = x*mu - f
  double pi  
    //= x * log(x) -    x  * log(1-x) - 2*x*x *chi
    //- x * log(x) - (1-x) * log(1-x) +   x*x *chi
    = - log(1-x) - x*x*chi;
  return(pi);
}
//=========================================================================
int multi_fun (   double* &x, double* &f, const int &N )
{
  f[0] = mu(x[0]) - mu(x[1]);
  f[1] = pi(x[0]) - pi(x[1]);
  return( 0 );
}
//=========================================================================
int Jacobi_Matrix( double* x, double** J, MULTI_FUN fun, int N )
{
  //double u = 2.013;
  double chi = CHI;
  // J[i][j] // i: runs for functions, j: runs for variables
  // J00= dmu/dx0
  // J01= dmu/dx1
  // J10= dpi/dx0
  // J11= dpi/dx1

  // mu     = log(x) - log(1-x) - 2*x*chi
  // pi     = - log(1-x) - x*x*chi

  // dmu/dx = 1/x + 1/(1-x) - 2*chi
  // dpi/dx = 1/(1-x) - 2*x*chi
  
  J[0][0] = + ( 1.0/x[0] + 1.0/(1.0-x[0]) - 2*chi );
  J[0][1] = - ( 1.0/x[1] + 1.0/(1.0-x[1]) - 2*chi );
  J[1][0] = + ( 1.0/(1.0-x[0]) - 2*x[0]*chi );
  J[1][1] = - ( 1.0/(1.0-x[1]) - 2*x[1]*chi );
  
  return(0);
}
//=========================================================================
int Write_Result( double* &x, double* &f, const int &N, const double &eps )
{
  int i,j;
  printf("\n***********************\n");
  for( j=0; j<N; j++ ){
    printf("x[%d]=%17.17e ",j,x[j] );
  }
  printf("\n");
  for( j=0; j<N; j++ ){
    printf("F[%d]=%17.17e ",j,f[j] );
  }
  printf("\n***********************\n");
  printf("This is solution ! for eps = %e", eps );
  printf("\n");
  return(0);
}
//=========================================================================
