
#include "multi_qnm.h"

int quasi_Newton_method( IOParameterSet &params,
			 double*        &x, 
			 MULTI_FUN       func,
			 JACOBI_FUN      Jacobi_func
			 )
{
  static double ***H, **J, *p, *q, *dx, *f;
  static bool first_flag = true;
  static int N = params.int_value("QUASI_NEWTON:N");
  static double eps = params.value("eps");
  
  if( first_flag == true ){
    ALLOC_3D(  H, N, N, N );
    ALLOC_2D(  J, N, N );
    ALLOC_1D(  p, N );
    ALLOC_1D(  q, N );
    ALLOC_1D(  f, N );
    ALLOC_1D( dx, N );
    first_flag = false;
  }

  for(int i=0;i<N;i++){ for(int j=0;j<N;j++){ H[0][i][j]=0.0; H[1][i][j]=0.0;}}
  
  Jacobi_func( params, x, multi_func, J, N );

  GJE0( J, N );

  for( int j=0; j<N; j++ ){ 
    for( int i=0; i<N; i++ ){
      H[0][i][j] = J[i][j];
    }
  }

  int step=0, NOW=0, NEW=1;
  func( params, x, f, N );

  while(1){

    for( int j=0; j<N; j++ ){ 
      dx[j] = 0.0;
      for( int i=0; i<N; i++ ){ 
	dx[j] -= H[NOW][j][i]*f[i];
      }
    }

    double min = 1.0e10;
    for( int j=0; j<N; j++ ){ 
      x[j] += dx[j];
      if( fabs(dx[j]) < min ){ min = fabs(dx[j]); }
    }
    if( min < eps ){ 
      printf("\n *********************** \n");
      for( int j=0; j<N; j++ ){	printf("x[%d]=%e \t", j, x[j]);  }
      printf("\n");
      for( int j=0; j<N; j++ ){ printf("f[%d]=%e \t", j, f[j] ); }
      printf("\n *********************** \n");
      printf("This is solution ! for eps = %e \n", eps);
      return(0);
    }

    func( params, x, f, N );

    double ss = 0.0, sHF = 0.0 ;
    for( int i=0; i<N; i++ ){ 
      ss += dx[i]*dx[i];
      for( int j=0; j<N; j++ ){
	sHF += dx[i]*H[NOW][i][j]*f[j];
      }
    }

    double inv__ss_plus_sHF = 1.0/(ss+sHF);    

    //printf("ss =%e \n", ss  );
    //printf("sHF=%e \n", sHF );
    //printf("inv__ss_plus_sHF = %e \n", inv__ss_plus_sHF );

    for( int i=0; i<N; i++ ){ 
      q[i] = 0.0; p[i] = 0.0;
      for( int j=0; j<N; j++ ){
	q[i] +=       H[NOW][i][j]*f[j];
	p[i] += dx[j]*H[NOW][j][i];
      }
    }

    for( int j=0; j<N; j++ ){ 
      for( int i=0; i<N; i++ ){ 
	H[NEW][i][j] = H[NOW][i][j] - q[i]*p[j] * inv__ss_plus_sHF;
      }
    }

    NOW = NEW;
    NEW = (NEW+1)%2;
  }
  return(0);
}
//=========================================================================
int Jacobi_Matrix(
		  IOParameterSet& params,
		   double*        x0,
		   MULTI_FUN      func,
		   double       **J,
		   const int     &N
		   )
{
  double dx     = 1.0e-3;
  double inv_dx = 1.0/dx;
  // J[i][j] // i: runs for functions, j: runs for variables

  static double *x1, *fo, *fd;
  static bool first_flag = true;
  
  if( first_flag == true ){
    ALLOC_1D( x1, N );
    ALLOC_1D( fo, N );
    ALLOC_1D( fd, N );
  }

  func( params, x0, fo, N );

  for( int j=0; j<N; j++ ){

    // delta(,): Kronecker's delta
    for( int jj=0; jj<N; jj++ ){ x1[jj]=x0[jj]+delta(j,jj)*dx; }

    func( params, x1, fd, N );
    
    for( int i=0; i<N; i++ ){
      J[i][j] = (fd[i]-fo[i])*inv_dx;
      //printf("J[%d][%d]=%e \n", i, j, J[i][j] );
    }
  }

  return(0);
}
//=========================================================================
