#include <math.h>
//#include "BICGSTAB.h"
#include <iostream.h>

typedef int (*MULTI_FUN)( double* x, double* f, int N );
typedef int (*JACOBI_FUN)( double* x0, MULTI_FUN fun, double (*J)[2], int N );

int multi_fun( double* x, double* f, int N );
int Jacobi_Matrix( double* x0, MULTI_FUN fun, double (*J)[2], int N );

int quasi_Newton_method( double* x0, 
			 double (*J)[2],
			 MULTI_FUN Multi_fun, 
			 JACOBI_FUN J_fun,
			 int N
			 );
//=========================================================================
int main()
{
  int N=2;
  double x0[]={3.0,3.0};
  double J[2][2];

  quasi_Newton_method( x0, J, multi_fun, Jacobi_Matrix, N );

}
//=========================================================================
int quasi_Newton_method( double* x, 
			 double (*J)[2], 
			 MULTI_FUN fun,
			 JACOBI_FUN Jacobi_fun,
			 int N )
{
  int i,j;
  double eps = 1.0e-5;
  double H[2][2][2];
  double q[2], p[2]; 

  for( j=0; j<N; j++ ){ 
    for( i=0; i<N; i++ ){ 
      H[0][i][j]=0.0;
      H[1][i][j]=0.0;
    }
  }

  double f[2];
  double dx[2];
  Jacobi_fun( x, multi_fun, J, N );

  double det = J[0][0]*J[1][1]-J[0][1]*J[1][0];
  cout << "det=" << det << endl;
  int g; cin >> g;
  double inv_det = 1.0/(J[0][0]*J[1][1]-J[0][1]*J[1][0]);
  H[0][0][0] = + J[1][1]*inv_det;
  H[0][1][1] = + J[0][0]*inv_det;
  H[0][0][1] = - J[0][1]*inv_det;
  H[0][1][0] = - J[1][0]*inv_det;

  int step=0, NOW=0, NEW=1;
  fun( x, f, N );
  cout << "f[0]=" << f[0] 
       << "f[1]=" << f[1] << endl;

  while(1){

    for( j=0; j<N; j++ ){ 
      dx[j] = 0.0;
      for( i=0; i<N; i++ ){ 
	dx[j] -= H[NOW][j][i]*f[i];
      }
    }

    double min = 1.0e10;
    for( j=0; j<N; j++ ){ 
      x[j] += dx[j]; 
      cout << x[j] << " " ;
      if( fabs(dx[j]) < min ){ min = fabs(dx[j]); }
    }
    if( min < eps ){ 
      for( j=0; j<N; j++ ){ 
	cout << x[j] << " " ;
      }
      cout << endl;
      cout << "This is solution ! for eps = " << eps << endl;
    }
    cout << endl;
    cin >> g;

    fun( x, f, N );

    double ss = 0.0, sHF = 0.0 ;
    for( i=0; i<N; i++ ){ 
      ss += dx[i]*dx[i];
      for( j=0; j<N; j++ ){
	sHF += dx[i]*H[NOW][i][j]*f[j];
      }
    }

    cout << "ss  = " << ss  << endl;
    cout << "sHF = " << sHF << endl;
    
    double inv__ss_plus_sHF = 1.0/(ss+sHF);
    cout << "inv__ss_plus_sHF = " << inv__ss_plus_sHF << endl;
    

    for( i=0; i<N; i++ ){ 
      q[i] = 0.0; p[i] = 0.0;
      for( j=0; j<N; j++ ){
	q[i] +=       H[NOW][i][j]*f[j];
	p[i] += dx[j]*H[NOW][j][i];
      }
    }

    for( j=0; j<N; j++ ){ 
      for( i=0; i<N; i++ ){ 
	H[NEW][i][j] = H[NOW][i][j] - q[i]*p[j] * inv__ss_plus_sHF;
      }
    }

    cout.precision(15);

    NOW = NEW;
    NEW = (NEW+1)%2;
  }
  return(0);
}
//=========================================================================
int multi_fun( double* x, double* f, int N )
{
  //f[0] = x[0]*x[0] + (x[1]-1.0)*(x[1]-1.0);
  //f[1] = (x[0]-3.0)*(x[0]-3.0) + (x[1]-2.0)*(x[1]-2.0);

  f[0] = x[0]*x[0] + x[1]*x[1] -5.0;
  f[1] = x[0]+2.0*x[1];
  return( 0 );
}
//=========================================================================
int Jacobi_Matrix( double* x0, MULTI_FUN fun, double (*J)[2], int N )
{
  int i,j;
  double dx = 1.0e-2;
  // J[i][j] // i: runs for functions, j: runs for variables

  double x1[2];
  double fo[2];
  double fd[2];
  double inv_dx  = 1.0/dx;

  fun( x0, fo, N );

  for( j=0; j<N; j++ ){
    for( int jj=0; jj<N; jj++ ){
      if( jj==j ) { x1[jj] = x0[jj] + dx; 
      }else{        x1[jj] = x0[jj]; }
      cout << "x1[" << jj << "]=" << x1[jj] << endl;
    }

    fun( x1, fd, N );
    for( i=0; i<N; i++ ){
      J[i][j] = (fd[i]-fo[i])*inv_dx;
      cout << "J[" << i << "][" << j << "]=" << J[i][j] << endl;
    }
  }
  //  int g; cin >> g;
  return(0);
}
//=========================================================================
